const JWT = require('../functions/JsonWebToken');

class Auth {

  isValidToken(token) {
    const verification = JWT.verify(token);
    return token && verification.valid !== false;
  }

  isAuth(req, res, next){
    const token = req.headers.authorization || '';
    const isAuthorizedRoute = this.checkRouteAuthorized(req.path);

    if (isAuthorizedRoute || this.isValidToken(token)) {
      return next();
    }

    const message = token ? 'UNAUTHORIZED_INVALID_TOKEN' : 'NOT_AUTHORIZED_EMPTY_TOKEN';

    res.status(401).send(message);
  }

  checkRouteAuthorized(route) {
    const authorizedRoutes = this.getAuthorizedRoutes().concat(this.getScreenRoutes());
    const isPublicPath = this.isPublicPath(route);
    return isPublicPath || this.isUpdaterRoute(route) || authorizedRoutes.some((authorized) => route === authorized || route === `${authorized}/`);
  }

  isUpdaterRoute(route) {
    return route.includes('/updater/');
  }

  isPublicPath(route) {
    return route.includes('/img/') ||
           route.includes('/metadata/') ||
           route.includes('/precache') ||
           route.includes('/assets/') || 
           route.includes('/API/');
  }

  getScreenRoutes() {
    return [
      '/dashboard',
      '/monitoring',
      '/monitoring-server',
      '/parametrization',
      '/active-version',
      '/up-version',
      '/system-management-by-unit',
      '/system-management-by-casher',
      '/service-management',
    ];
  }

  getAuthorizedRoutes() {
    return [
      '/',
      '/index.html',
      '/manifest.json',
      '/service-worker.js',
      '/robots.txt',
      '/favicon.ico',
      '/login'
    ];
  }
}

module.exports = new Auth()