const OracleService = require('../service/OracleService');

class updaterModelCasher {
    constructor(){
        this.GET_INSTALLED_VERSION = `
            SELECT 
                *
            FROM
                UPDATERPRODFILICX 
            WHERE
                CDFILIAL = :P_CDFILIAL AND
                CDCAIXA  = :P_CDCAIXA AND
                UPDATERPRODID = :P_UPDATERPRODID AND
                (NRVERSAOATUAL = :P_NRVERSAOATUAL OR (:P_NRVERSAOATUAL IS NULL))       
        `;

        this.INSERT_EVENT = `
            INSERT INTO UPDATEREVENTOSCX (
                CDFILIAL,
                CDCAIXA,
                CDEVENTO,
                CDUUID,
                CDPRODUCTID,
                CDPRODFILIID,
                IDSTEVENTO,
                NRVERSAOATUAL,
                NRVERSAOALVO,
                IDCRIARVERSAO,
                DTINCLUSAO,
                DTULTATU
            ) VALUES (
                :P_CDFILIAL,
                :P_CDCAIXA,
                :P_CDEVENTO,
                :P_CDUUID,
                :P_CDPRODUCTID,
                :P_CDPRODFILIID,
                :P_IDSTEVENTO,
                :P_NRVERSAOATUAL,
                :P_NRVERSAOALVO,
                :P_IDCRIARVERSAO,
                SYSDATE,
                SYSDATE
            )
        `;

        this.INSERT_HISTORY = `
            INSERT INTO UPDATERHISTORICOCX (
                CDFILIAL,
                CDCAIXA,
                CDHISTORICO,
                CDEVENTO,
                CDOPERADOR,
                CDUUID,
                CDPRODUCTID,
                NMPRODUCT,
                IDSTEVENTO,
                NRVERSAOATUAL,
                NRVERSAOALVO,
                IDCRIARVERSAO,
                DTINCLUSAO,
                DTULTATU
            ) VALUES (
                :P_CDFILIAL,
                :P_CDCAIXA,
                :P_CDHISTORICO,
                :P_CDEVENTO,
                :P_CDOPERADOR,
                :P_CDUUID,
                :P_CDPRODUCTID,
                :P_NMPRODUCT,
                :P_IDSTEVENTO,
                :P_NRVERSAOATUAL,
                :P_NRVERSAOALVO,
                :P_IDCRIARVERSAO,
                SYSDATE,
                SYSDATE
            )
        `;

        this.GET_EVENTS = `
            SELECT 
                * 
            FROM 
                UPDATEREVENTOSCX 
            WHERE 
                CDFILIAL = :P_CDFILIAL AND
                CDCAIXA  = :P_CDCAIXA  AND
                IDSTEVENTO IN (
                    'INSTALACAO_PENDENTE', 
                    'REMOCAO_PENDENTE', 
                    'INSTALACAO_EM_CURSO', 
                    'REMOCAO_EM_CURSO'
                )
        `;

        this.GET_EVENTS_BY_PRODUCT_AND_EVENT_ID = `
            SELECT 
                * 
            FROM 
                UPDATEREVENTOSCX
            WHERE 
                CDFILIAL = :P_CDFILIAL AND 
                CDCAIXA  = :P_CDCAIXA  AND
                CDPRODUCTID = :P_CDPRODUCTID AND 
                IDSTEVENTO = :P_IDSTEVENTO
        `

        this.GET_IN_PROGRESS_EVENTS = `
            SELECT 
                * 
            FROM 
                UPDATEREVENTOSCX 
            WHERE 
                CDFILIAL = :P_CDFILIAL AND 
                CDCAIXA  = :P_CDCAIXA  AND
                IDSTEVENTO IN ('INSTALACAO_EM_CURSO', 'REMOCAO_EM_CURSO')
        `;

        this.UPDATE_EVENT_STATUS = `
            UPDATE UPDATEREVENTOSCX
            SET
                IDSTEVENTO = :P_IDSTEVENTO,
                DTULTATU = SYSDATE
            WHERE
                CDEVENTO = :P_CDEVENTO
        `;

        this.GET_EVENT_HISTORY = `
            SELECT
                *
            FROM
                UPDATERHISTORICOCX
            WHERE
                CDEVENTO   = :P_CDEVENTO AND
                IDSTEVENTO = :P_IDSTEVENTO
        `;

        this.UPDATE_PRODFILI_VERSION = `
            UPDATE 
                UPDATERPRODFILICX
            SET
                NRVERSAOATUAL = :P_NRVERSAO,
                DTULTATU      = SYSDATE
            WHERE
                ID = :P_ID
        `;

        this.CANCEL_UNIT_EVENTS = `
            UPDATE 
                UPDATEREVENTOSCX
            SET
                IDSTEVENTO = 'CANCELADO',
                DTULTATU = SYSDATE
            WHERE
                CDFILIAL = :P_CDFILIAL AND
                ((CDCAIXA = :P_CDCAIXA) OR (:P_CDCAIXA IS NULL))
        `;

        this.GET_UPDATER_PENDING_EVENTS = `
            SELECT 
                UE.CDFILIAL,
                UE.CDCAIXA,
                UPH.CDOPERADOR,
                UE.CDEVENTO,
                UE.CDPRODUCTID,
                UE.CDPRODFILIID,
                UE.IDSTEVENTO,
                UE.NRVERSAOATUAL,
                UE.NRVERSAOALVO,
                UE.IDCRIARVERSAO,
                UE.DTINCLUSAO,
                UPV.DSDIRETORIO,
                UPV.IDDOWNLOADVER,
                UPC.CONFIG       
            FROM 
                UPDATEREVENTOSCX UE
                LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAOALVO = UPV.NRVERSAO)
                LEFT JOIN UPDATERPRODCONFIG UPC ON (UE.CDPRODUCTID = UPC.CDPRODUCTID AND UE.NRVERSAOALVO = UPC.NRVERSAO)
                LEFT JOIN UPDATERHISTORICOCX UPH ON (UE.CDEVENTO = UPH.CDEVENTO)          
            WHERE 
                UE.CDFILIAL = :P_CDFILIAL AND
                UE.CDCAIXA = :P_CDCAIXA AND
                UE.IDSTEVENTO IN ('INSTALACAO_PENDENTE', 'INSTALACAO_EM_CURSO')
            UNION
            SELECT 
                UE.CDFILIAL,
                UE.CDCAIXA,
                UPH.CDOPERADOR,
                UE.CDEVENTO,
                UE.CDPRODUCTID,
                UE.CDPRODFILIID,
                UE.IDSTEVENTO,
                UE.NRVERSAOATUAL,
                UE.NRVERSAOALVO,
                UE.IDCRIARVERSAO,
                UE.DTINCLUSAO,
                UPV.DSDIRETORIO,
                UPV.IDDOWNLOADVER,
                '' AS CONFIG
            FROM 
                UPDATEREVENTOSCX UE
                LEFT JOIN UPDATERPRODVERSAO UPV ON (UE.CDPRODUCTID = UPV.UPDATERPRODID AND UE.NRVERSAOATUAL = UPV.NRVERSAO)
                LEFT JOIN UPDATERHISTORICOCX UPH ON (UE.CDEVENTO = UPH.CDEVENTO)
            WHERE 
                UE.CDFILIAL = :P_CDFILIAL AND 
                UE.CDCAIXA = :P_CDCAIXA AND
                UE.IDSTEVENTO IN ('REMOCAO_PENDENTE', 'REMOCAO_EM_CURSO')
        `;

        this.GET_VERSION = `
            SELECT 
                ID
            FROM
                UPDATERPRODFILICX 
            WHERE
                CDFILIAL = :P_CDFILIAL AND
                CDCAIXA  = :P_CDCAIXA  AND
                UPDATERPRODID = :P_UPDATERPRODID AND
                NRVERSAOATUAL = :P_NRVERSAOATUAL
        `;

        this.GET_PRODUCTS_VERSION = `
            SELECT
                PF.ID AS PRODFILIID,
                F.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
                F.CDFILIAL,
                F.NMFILIAL,
                CX.CDCAIXA,
                CX.NMCAIXA,
                CX.CDCAIXA || ' - ' || CX.NMCAIXA AS CAIXA,
                P.ID AS PRODUCTID,
                P.NMPRODUCT,
                EV.CDEVENTO,
                CASE
                WHEN PF.NRVERSAOATUAL IS NULL THEN
                    'INSTALACAO_PENDENTE'
                ELSE
                    'INSTALADO'
                END AS IDSTEVENTO,
                DECODE(PF.NRVERSAOATUAL, NULL, '-', PF.NRVERSAOATUAL) AS NRVERSAOATUAL,
                TO_CHAR(PF.DTULTATU, 'DD/MM/YYYY HH24:MI') AS DTULTATU,
                OP.CDOPERADOR,
                OP.NMOPERADOR
            FROM
                UPDATERPRODFILICX PF 
                INNER JOIN UPDATERPRODUTOS P 
                    ON (PF.UPDATERPRODID = P.ID)
                INNER JOIN FILIAL F 
                    ON (PF.CDFILIAL = F.CDFILIAL)
                INNER JOIN CAIXA CX
                    ON PF.CDFILIAL = CX.CDFILIAL
                    AND PF.CDCAIXA = CX.CDCAIXA
                LEFT JOIN (
                    SELECT
                        CDFILIAL,
                        CDCAIXA,
                        CDPRODFILIID,
                        CDPRODUCTID,
                        MAX(CDEVENTO) AS CDEVENTO
                    FROM
                        UPDATEREVENTOSCX
                    WHERE
                        IDSTEVENTO = 'FINALIZADO'
                    GROUP BY
                        CDFILIAL,
                        CDCAIXA,
                        CDPRODUCTID,
                        CDPRODFILIID
                ) EV
                    ON (
                        EV.CDPRODUCTID = P.ID AND 
                        EV.CDPRODFILIID = PF.ID AND 
                        EV.CDFILIAL = PF.CDFILIAL AND 
                        EV.CDCAIXA  = PF.CDCAIXA
                    )
                LEFT JOIN UPDATERHISTORICOCX HIST 
                    ON  HIST.CDEVENTO = EV.CDEVENTO
                    AND HIST.IDSTEVENTO = 'FINALIZADO'
                LEFT JOIN OPERADOR OP 
                    ON OP.CDOPERADOR = HIST.CDOPERADOR
            WHERE
                PF.CDFILIAL = :P_CDFILIAL AND
                PF.CDCAIXA  = :P_CDCAIXA
            ORDER BY
                P.ID ASC
        `;
    }

    async getInstalledVersion(params) {
        return OracleService.fetchAssoc(this.GET_INSTALLED_VERSION, params);
    };

    async getEvents(params) {
        return OracleService.fetchAssoc(this.GET_EVENTS, params);
    };

    async getUpdaterPendingEvents(params) {
        return OracleService.fetchAssoc(this.GET_UPDATER_PENDING_EVENTS, params);
    }

    async getEventsInProgress(params) {
        return OracleService.fetchAssoc(this.GET_IN_PROGRESS_EVENTS, params);
    }

    async getEventsByIdAndProduct(params) {
        return OracleService.fetchAssoc(this.GET_EVENTS_BY_PRODUCT_AND_EVENT_ID, params);
    }

    async getEventHistory(params) {
        return OracleService.fetchAssoc(this.GET_EVENT_HISTORY, params);
    }

    async insertEvent(params) {
        return OracleService.execute(this.INSERT_EVENT, params);
    };

    async insertHistory(params) {
        return OracleService.execute(this.INSERT_HISTORY, params);
    };

    async updateEventStatus(params) {
        return OracleService.execute(this.UPDATE_EVENT_STATUS, params);
    }

    async updateProdFiliVersion(params) {
        return OracleService.execute(this.UPDATE_PRODFILI_VERSION, params);
    }

    async cancelUnitEvents(params) {
        return OracleService.execute(this.CANCEL_UNIT_EVENTS, params);
    }

    async getVersion(params) {
        return OracleService.fetchAssoc(this.GET_VERSION, params);
    }

    async getProductsVersion(params) {
        return OracleService.fetchAssoc(this.GET_PRODUCTS_VERSION, params);
    }
}

module.exports = new updaterModelCasher();